package com.moonapps.commandroid;

/**
 * Copyright (C) 2012 Daniel Pavon Perez
 *  
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *  
 *      http://www.apache.org/licenses/LICENSE-2.0
 *  
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

import com.moonapps.commandroid.agents.Constants;
import com.moonapps.commandroid.profiles.Action;

import android.app.AlarmManager;
import android.app.PendingIntent;
import android.content.Context;
import android.content.Intent;
import android.net.wifi.WifiManager;
import android.net.wifi.WifiManager.WifiLock;
import android.os.Bundle;
import android.os.PowerManager;
import android.os.PowerManager.WakeLock;
import android.util.Log;

public class WifiActionManager {

	private static final int WIFI_MODE_FULL_HIGH_PERF = 3;
	private static final int TIMEON = 5;
	private boolean isTimerActivated = false;
	private WifiLock lock;
	//private int lockType;
	
	Context context;
	private WakeLock wakeLock;
	//private Timer timer;
	private int minutes;
	//private Timer timer2;
	private PendingIntent pintentOn;
	private PendingIntent pintentOff;
	
	public WifiActionManager (Context context){
		
		this.lock=null;
	    this.context=context;
		//this.lockType = -1;
	}
	
	
	public void processAction(Action action){
		int value=-1;
		
		if (action!=null){
			value=action.getValue();
		}
		
		switch (value){
		
		case 0: //WIFI OFF
			this.cancelTimer();
			this.modeOff();
			break;
		case 1: //WIFI ON
			this.cancelTimer();
			this.modeOn();
			break;
		case 2: //WIFI KEEP ON - NORMAL POWER
			this.cancelTimer();
			this.modeKeepOnNormal();
			break;
		case 3: //WIFI KEEP ON - FULL POWER
			this.cancelTimer();
			this.modeKeepOnFull();
			break;		
		default: 
			if (this.isTimerActivated==false){
				this.releaseAllLocks();
				this.setTimers(value);
			}else{
				LogManager.print(Log.VERBOSE, Constants.debugTag, "WIFACTMANAG: Ignored.");
			}
			break;
		}
		
		
		LogManager.print(Log.DEBUG, Constants.debugTag, "***WifiActionManager: ValueReceived"+value);
	}
	
	


	private void modeOff() {
		this.releaseAllLocks();
		this.setWifi(false);		
	}


	private void modeOn() {
		this.releaseAllLocks();
		this.setWifi(true);		
	}


	private void modeKeepOnNormal() {
	
		this.releaseAllLocks();
		this.setWifi(true);
		this.acquireCPULock();
		this.acquireLock(WifiManager.WIFI_MODE_FULL);		
	}


	private void modeKeepOnFull() {
		this.releaseAllLocks();
		this.setWifi(true);
		this.acquireCPULock();
		//if (android.os.Build.VERSION.SDK_INT >= 12) {
			this.acquireLock(WIFI_MODE_FULL_HIGH_PERF);
	/*	}else{
			this.acquireLock(WifiManager.WIFI_MODE_FULL);
		}*/		
	}


	private void releaseLock() {
		if (lock!=null)
			if (this.lock.isHeld())
				lock.release();
		//this.lockType = -1;
		this.lock = null;		
	}
	
	public void acquireLock(int type) {
		if (lock==null){
			//this.lockType=type;
			WifiManager wifiManager = (WifiManager) this.context.getSystemService(Context.WIFI_SERVICE);
			this.lock = wifiManager.createWifiLock(type, Constants.debugTag);
			this.lock.acquire();
		}
	}
	
	
	
	public void acquireCPULock(){
		if (wakeLock==null){
			this.wakeLock = ((PowerManager) context.getSystemService(Context.POWER_SERVICE))
			.newWakeLock(PowerManager.PARTIAL_WAKE_LOCK, Constants.debugTag+": WakeLock");
			this.wakeLock.acquire();
			// TODO: LONG ACQUIRE? Quizas sea util para temporizador.
		}
	}
	private void releaseCPULock(){
		if (this.wakeLock!=null){
			if (this.wakeLock.isHeld())
				this.wakeLock.release();
			this.wakeLock=null;
		}
	}

	
	

	public void setWifi(boolean enabled){
		LogManager.print(Log.DEBUG, Constants.debugTag, "***WifiActionManager: SetWifi-"+enabled);

		
			  WifiManager wifiManager = (WifiManager) this.context.getSystemService(Context.WIFI_SERVICE);
		
			  if(wifiManager.isWifiEnabled()!=enabled){
			       wifiManager.setWifiEnabled(enabled);
			  }
	}
	
	public void releaseAllLocks(){
		this.releaseLock();
		this.releaseCPULock();
		
	}
	
	public void releaseEverything(){
		this.releaseAllLocks();
		this.cancelTimer();
		LogManager.print(Log.ERROR, Constants.debugTag, "WIFACTMAN: Releasing everything.");
	}


	private void setTimers(int value) {
		// TODO Colocarle un valor al reloj que regula los tiempos muertos, y al activo de 5 min.
		
		if (this.isTimerActivated==false){
			LogManager.print(Log.VERBOSE, Constants.debugTag, "WIFACTMANAG: Start of timer.");
			this.isTimerActivated=true;

		
			if (value<10){ 
				LogManager.print(Log.VERBOSE, Constants.debugTag, "WIFACTMANAG: Low value. Modifying");
				value=10;
				}
			
			
			/*if (this.timer!=null) 
			this.timer.cancel();
			else
			this.timer = new Timer();*/
			
			this.minutes = value;
			this.fiveMinutesOn();
		}else{}

		
		
	}


	public void fiveMinutesOn() {
		LogManager.print(Log.VERBOSE, Constants.debugTag, "WIFACTMANAG: Five minutes on.");
		this.modeKeepOnFull();
		

		this.setAlarmOff(System.currentTimeMillis()+1000*60*WifiActionManager.TIMEON);
		
		/*TimerTask task = new TimerTask() {
			
	        @Override
	        public void run() {
	            LogManager.print(Log.VERBOSE, Constants.debugTag, "WIFACTMANAG: Turn Wifi off.");
	            modeOff();
	            restartTimer();

	        }
	
	    };
	    if (timer!=null)
	    	timer.schedule(task, new Date(System.currentTimeMillis()+1000*60*WifiActionManager.TIMEON));
	    else{
	    	LogManager.print(Log.ERROR, Constants.debugTag, "WIFIACTMANAG:  TIMER ERROR.");
	    }*/
		
	}


	public void offlineTime() {
		
		if (this.minutes>=(WifiActionManager.TIMEON*2)){
			
			int schedule = this.minutes-WifiActionManager.TIMEON;
			
			long next = System.currentTimeMillis()+1000*60*schedule;
			
			this.modeOff();
			this.setAlarmOn(next);
			
			LogManager.print(Log.VERBOSE, Constants.debugTag, "WIFACTMANAG: "+schedule+" minutes of.");

			/*
			LogManager.print(Log.VERBOSE, Constants.debugTag, "WIFACTMANAG: Restarting timer.");
			
			if (this.timer2!=null) 
				this.timer2.cancel();
			else{
				this.timer2 = new Timer();}		
			TimerTask task = new TimerTask() {
			
				@Override
	        	public void run() {
					LogManager.print(Log.VERBOSE, Constants.debugTag, "WIFACTMANAG: Restarted Wifi.");
	            	fiveMinutesOn();
	        	}
	
			};
			
			LogManager.print(Log.VERBOSE, Constants.debugTag, "WIFACTMAN: NEXT TIMER:"+next.toLocaleString()+"because of schedule:"+schedule );
			timer2.schedule(task, next);*/
		}
		else{
			LogManager.print(Log.ERROR, Constants.debugTag, "WIFACTMAN: TIMING ERROR. Minutes<TIMEON*2" );
		}
	}


	private void setAlarmOn(long time) {
		
		Intent intent = new Intent(Constants.ACTION_SEND_ALARM);
		

		Bundle b = new Bundle();
		b.putInt("commandroid.wifiState", Integer.valueOf(3));
		intent.putExtra("bundleWifi", b);
		
		//LogManager.print(Log.ERROR, Constants.debugTag, "Creating on alarm for: "+new Date(time).toLocaleString());
		
		this.pintentOn = PendingIntent.getBroadcast(
				this.context, Constants.wifiAlarm, 
                intent, PendingIntent.FLAG_UPDATE_CURRENT);

		AlarmManager alarm = 
			(AlarmManager) this.context.getSystemService(Context.ALARM_SERVICE);

		alarm.set(AlarmManager.RTC_WAKEUP, 
				time, this.pintentOn);		
	}
	
	private void setAlarmOff(long time) {
		
		Intent intent = new Intent(Constants.ACTION_SEND_ALARM);
		
		Bundle b = new Bundle();
		b.putInt("commandroid.wifiState", Integer.valueOf(2));
		intent.putExtra("bundleWifi", b);


		//LogManager.print(Log.ERROR, Constants.debugTag, "Creating off alarm for: "+new Date(time).toLocaleString());

		
		this.pintentOff = PendingIntent.getBroadcast(
				this.context, Constants.wifiAlarm, 
                intent, PendingIntent.FLAG_UPDATE_CURRENT);

		AlarmManager alarm = 
			(AlarmManager) this.context.getSystemService(Context.ALARM_SERVICE);

		alarm.set(AlarmManager.RTC_WAKEUP, 
				time, this.pintentOff);		
	}


	public void cancelTimer() {
		/*
		if (this.timer!=null){ 
			this.timer.cancel();
			this.timer=null;
			LogManager.print(Log.VERBOSE, Constants.debugTag, "WIFACTMANAG: Cancelling timer1.");
		}	
		if (this.timer2!=null){ 
			this.timer2.cancel();
			this.timer2=null;
			LogManager.print(Log.VERBOSE, Constants.debugTag, "WIFACTMANAG: Cancelling timer2.");
		}	*/
		
		if (this.pintentOn!=null){

			AlarmManager alarm = (AlarmManager) this.context.getSystemService(Context.ALARM_SERVICE);
			alarm.cancel(this.pintentOn);		
			
		}
		
		if (this.pintentOff!=null){

			AlarmManager alarm = (AlarmManager) this.context.getSystemService(Context.ALARM_SERVICE);
			alarm.cancel(this.pintentOff);		
			
		}
		
		this.isTimerActivated=false;
	}

	
}


