package com.moonapps.commandroid.adapters;

/**
 * Copyright (C) 2012 Daniel Pavon Perez
 *  
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *  
 *      http://www.apache.org/licenses/LICENSE-2.0
 *  
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

import java.util.ArrayList;

import com.moonapps.commandroid.R;
import com.moonapps.commandroid.agents.Metric;
import com.moonapps.commandroid.agents.MetricsManager;
import com.moonapps.commandroid.listeners.EditTriggerListener;
import com.moonapps.commandroid.listeners.SelectOperatorListener;
import com.moonapps.commandroid.listeners.ValueContListener;
import com.moonapps.commandroid.listeners.ValueDiscreteListener;
import com.moonapps.commandroid.profiles.Trigger;

import android.content.Context;
import android.view.LayoutInflater;
import android.view.View;
import android.view.View.OnClickListener;
import android.view.ViewGroup;
import android.widget.ArrayAdapter;
import android.widget.ImageView;
import android.widget.ListView;
import android.widget.RelativeLayout;
import android.widget.TextView;

public class TriggerAdapter extends ArrayAdapter<Trigger> implements OnClickListener{

	Context context;
	ArrayList<Trigger> objects;
	
	public TriggerAdapter(Context context, int resource, ArrayList<Trigger> objects) {
		super(context, resource, objects);
		this.context=context;
		this.objects=objects;
		}
 

	@Override
    public View getView(int position, View convertView, ViewGroup parent) {
		return this.getNewView (position, convertView, parent);                		
	
	}


	private View getNewView(int position, View convertView, ViewGroup parent) {

		Trigger trigg = this.getItem(position);
		View view = new View(this.context);

		if (trigg != null) {
			
			int id = trigg.getTriggerID();
			Metric chosenMetric = MetricsManager.metrics.get(id);

			
			LayoutInflater inflater = (LayoutInflater) this.context
					.getSystemService(Context.LAYOUT_INFLATER_SERVICE);

			//INFLATE LAYOUT. 2 TYPES.
			if (!chosenMetric.isDiscrete()) {
				view = inflater.inflate(R.layout.listelementtrigger, parent,
						false);
				this.prepareOperator(view, position, trigg);
			} else {
				view = inflater.inflate(R.layout.listelementtrigger_noop,
						parent, false);
			}

			
			this.prepareNameButton(view, position, chosenMetric);
			this.prepareValueButton(view,chosenMetric, trigg, position);
			this.prepareDeleteButton(view);
	
		}

		return view;
	}


	private void prepareOperator(View view, int position, Trigger trigg) {
		TextView operator = (TextView) view.findViewById(R.id.operator);
		operator.setText(MetricsManager.getOperatorString(trigg
				.getOperator()));
		operator.setOnClickListener(new SelectOperatorListener(
				this.context, position, this.objects, this));		
	}


	private void prepareNameButton(View view, int position, Metric chosenMetric) {
		// NAME
		TextView triggername = (TextView) view
				.findViewById(R.id.triggername);
		triggername.setOnClickListener(new EditTriggerListener(
				this.context, position, this, objects));	

		// SET ID TEXT
		int stringID = (int) chosenMetric.getStringID();
		String text = (String) this.context.getText(stringID);
		triggername.setText(text);
	}


	private void prepareValueButton(View view, Metric chosenMetric, Trigger trigg, int position) {
		// VALUE
		TextView value = (TextView) view.findViewById(R.id.value);
		if (chosenMetric.isDiscrete()) {
			int resourceID = chosenMetric.getValues().get(trigg.getValue());
			value.setText(resourceID);
			value.setOnClickListener(new ValueDiscreteListener(
					this.context, position, this, this.objects));
		} else {
			value.setText("" + trigg.getValue());
			value.setOnClickListener(
					new ValueContListener(this.context, position, this, objects));
		}		
	}


	private void prepareDeleteButton(View view) {
		// DELETEBUTTON
		ImageView deletetriggericon = (ImageView) view
				.findViewById(R.id.deletetriggericon);
		deletetriggericon.setOnClickListener(this);		
	}


	@Override
	public void onClick(View v) {
		
		
		//PELIGROSO. DEPENDIENTE DE CAMBIOS EN XML.
		
		
		ListView list =  (ListView) v.getParent().getParent();
		RelativeLayout parent = (RelativeLayout) v.getParent();
		int num = list.getChildCount();
		int position=-1;
		int i=0;
		for (i=0;i<num;i++)
			if (parent.equals(list.getChildAt(i)))
				position=i;
		
		if (position!=-1)
			this.remove(this.getItem(position));
		
	}

	
	

	
}
