package com.moonapps.commandroid.agents;

/**
 * Copyright (C) 2012 Daniel Pavon Perez
 *  
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *  
 *      http://www.apache.org/licenses/LICENSE-2.0
 *  
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

import java.io.FileNotFoundException;
import java.util.ArrayList;

import android.content.Context;
import android.util.Log;

import com.moonapps.commandroid.FileManager;
import com.moonapps.commandroid.LogManager;
import com.moonapps.commandroid.R;
import com.moonapps.commandroid.graph.BasicData;

public abstract class Agent{

	protected int limit;
	protected ArrayList<BasicData> data;
	protected String path;
	protected String classname;
	protected int resourceID;
	
	public Agent (){
		this.data=new ArrayList<BasicData>();
		this.classname="";
		this.limit=1024;
		this.resourceID=0;
	}
	
	public int saveData (Context ctx){
		FileManager fileM = FileManager.getInstance();
		int success=-1;
		try {
			fileM.saveData(ctx.openFileOutput(path, Context.MODE_PRIVATE), this.data);
			success=0;
		} catch (FileNotFoundException e) {
			LogManager.print(Log.ERROR, Constants.debugTag,ctx.getString(R.string.savefileerror));
			success=-1;		
		}
		return success;
	}
	 
	
	public int loadData (Context ctx){
			FileManager fileM = FileManager.getInstance();

			int success=-1;
	    try {
			fileM.loadData(ctx.openFileInput(this.path),this.data);
			success=0;
	        } catch (FileNotFoundException e) {
	        	LogManager.print(Log.ERROR, Constants.debugTag, this.classname+": "+ctx.getString(R.string.loadfileerror));
				success=-1;
	        }
	        return success;
		
	}
	
	public int processNewData(long time, int value){
		int dataAmount = 0;
		
		if (this.isFull())
			this.releaseData();
		
		
		Metric metric = MetricsManager.metrics.get(this.resourceID);
		boolean isDiscrete = false;
		if (metric==null){
			LogManager.print(Log.ERROR, Constants.debugTag, "Agent:"+this.classname+" Metric null, ID:"+this.resourceID);
		}else{
			isDiscrete = metric.isDiscrete();
		}
		
		
		//old isDiscrete(this.resourceID)
		if (this.isNewData(value) && isDiscrete==false){
			this.data.add(new BasicData(time,value));
			dataAmount=1;
		}		
		if (this.isNewData(value) && isDiscrete==true){
			if (this.data.size()>0)
				this.data.add(new BasicData(time-100,this.data.get(this.data.size()-1).getValue()));
			this.data.add(new BasicData(time,value));
			dataAmount=2;
		}		
		
		if (metric==null)
			dataAmount=0;
		else if (!metric.isTrigger() && metric.getId()!=Constants.REQUEST_SCREEN_LOCK)
			dataAmount=0;
		
		return dataAmount;
	}
	
	
	public boolean isNewData(int value){
			if (this.data.size()>0){
				BasicData last = this.data.get(this.data.size()-1);
				boolean isDifferentValue = last.getValue()!=value;
				boolean oldDataIsOld = (last.getTime()+60*60*1000<System.currentTimeMillis());
				return (isDifferentValue || oldDataIsOld);
			}else
				return true;	
	}
	
	public boolean isFull(){
		return (this.data.size()>this.limit);
	}
	public void releaseData(){
		if (this.isFull()){
			LogManager.print(Log.ERROR, Constants.debugTag,"  Agent- "+classname+" Limit reached, removing...");
			for (int i=0; i<50;i++){
				data.remove(0);
				}
		}		
		
	}
	
	public double getLastValue(){

		if (this.data.size()>0)
			return this.data.get(this.data.size()-1).getValue();
		else			
			return Integer.MIN_VALUE;
	}
	

	
	public void setData(ArrayList<BasicData> data) {
		this.data = data;
	}
	public ArrayList<BasicData> getData() {
		return data;
	}
	public void setPath(String path) {
		this.path = path;
	}
	public String getPath() {
		return path;
	}

	//protected boolean isDiscrete;
	
	/*public boolean isDiscrete(int id){
		return MetricsManager.isDiscrete(this.resourceID);
	}*/
	
	/*public double maximumValue(){
		return MetricsManager.getMaxValue(this.resourceID);
	}
	public double minimumValue(){
		return MetricsManager.getMinValue(this.resourceID);
	}*/
	
	public int getResourceID() {
		return resourceID;
	}

	public void setResourceID(int resourceID) {
		this.resourceID = resourceID;
	}
	
	
}
