package com.moonapps.commandroid.agents;

/**
 * Copyright (C) 2012 Daniel Pavon Perez
 *  
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *  
 *      http://www.apache.org/licenses/LICENSE-2.0
 *  
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.LinkedHashMap;

import com.moonapps.commandroid.LogManager;
import com.moonapps.commandroid.graph.BasicData;

import android.content.Context;
import android.content.Intent;
import android.os.Bundle;
import android.util.Log;

public abstract class Controller {
	
	protected String classname="Controller";
	public LinkedHashMap<Integer, Agent> agents;
	
	public Controller (){
		this.agents = new LinkedHashMap<Integer,Agent>();
		
	}
	abstract protected void initAgents();

	
	
	abstract public int processBundle(Bundle bundle);
	
	public int processRequest(Context context, int dataType, int graphNumber){
			
			ArrayList<BasicData> dataResult =this.requestAgentData(dataType);
			if (dataResult!=null){
				LogManager.print(Log.DEBUG, Constants.debugTag,classname+": Agents returned data.");
				this.sendData(context, dataResult, dataType, graphNumber);
			}
			return 0;
	}
		
		
	private ArrayList<BasicData> requestAgentData(int dataType) {
		ArrayList<BasicData> dataResult = null;
		Agent selected = this.agents.get(dataType);
		if (selected!=null)
			dataResult=selected.getData();
		return dataResult;
	}
	protected void sendData(Context context, ArrayList<BasicData> dataResult, int dataType, int graphNumber) {
		LogManager.print(Log.DEBUG, Constants.debugTag, this.classname+": Sending " + 
				dataResult.size()+" elements.");		
		
		//ActivityReceiver in Manifest.
		Intent toSend = new Intent(Constants.ACTION_DATARESPONSE);
		
		toSend.putExtra(Constants.ACTION_DATARESPONSE, Constants.RESPONSE);
		toSend.putExtra(Constants.EXTRAKEY_DATATYPE, dataType);
		toSend.putExtra(Constants.EXTRAKEY_GRAPHNUMBER, graphNumber);
		toSend.putParcelableArrayListExtra(Constants.EXTRAKEY_DATA, dataResult);
		context.sendBroadcast(toSend);
		
		
	}
	//LOAD AND SAVE.
	public int loadFile(Context ctx){
		
		Collection<Agent> col =this.agents.values();
		Iterator<Agent> it =col.iterator();
		int success=0;
		while (it.hasNext()){
			Agent tmp = (Agent) it.next();
			success = success+tmp.loadData(ctx);
		}
		return -success;
    }
	
	
	public int saveFile(Context ctx){
		
		Collection<Agent> col =this.agents.values();
		Iterator<Agent> it =col.iterator();
		int success=0;
		while (it.hasNext()){
			Agent tmp = (Agent) it.next();
			success = success+tmp.saveData(ctx);
		}
		return -success;
	}

	
}
