package com.moonapps.commandroid.dialogs;

/**
 * Copyright (C) 2012 Daniel Pavon Perez
 *  
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *  
 *      http://www.apache.org/licenses/LICENSE-2.0
 *  
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

import java.util.ArrayList;

import com.moonapps.commandroid.DataService;
import com.moonapps.commandroid.LogManager;
import com.moonapps.commandroid.R;
import com.moonapps.commandroid.agents.Constants;
import com.moonapps.commandroid.agents.Metric;
import com.moonapps.commandroid.agents.MetricsManager;
import com.moonapps.commandroid.graph.DataGraph;

import android.app.AlertDialog;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.util.Log;
import android.widget.Button;

public class DataDialog extends AlertDialog.Builder implements DialogInterface.OnClickListener {

		
	private DataGraph data;
	private Button button;
	private Context ctx;
	private int graphNumber;
	private ArrayList<Integer> ids;

	public DataDialog(Context ctx, int graphNumber){
		super(ctx);
		this.graphNumber=graphNumber;
		this.ctx=ctx;
		this.data = null;
		this.button=null;
  		this.setTitle(R.string.dialog_selectData);
		this.setItems(this.getItemsSequence(), this);
	}
	
	public void createDialog(DataGraph data, Button button){
		this.data=data;
		this.button=button;
		this.show();
	}
		
	
	private CharSequence[] getItemsSequence() {
		
		ArrayList<CharSequence> chars = new ArrayList<CharSequence>();
		this.ids = new ArrayList<Integer>();
		for (Metric e: MetricsManager.metrics.values()){
			this.ids.add(e.getId());
			chars.add(this.ctx.getText(e.getStringID()));
		}
			
		CharSequence[] dataChoices = new CharSequence[0];
		/*dataChoices[0]=this.ctx.getString(R.string.battery_health);
		dataChoices[1]=this.ctx.getString(R.string.battery_level);
		dataChoices[2]=this.ctx.getString(R.string.battery_plugged);
		dataChoices[3]=this.ctx.getString(R.string.battery_status);
		dataChoices[4]=this.ctx.getString(R.string.battery_temperature);
		dataChoices[5]=this.ctx.getString(R.string.connection_type);
		dataChoices[6]=this.ctx.getString(R.string.connection_subtype);
		dataChoices[7]=this.ctx.getString(R.string.screen_lock);
		 */
		dataChoices = chars.toArray(dataChoices);
		
		return dataChoices;
	}

	public void onClick(DialogInterface d, int choice) {
		int destination = Constants.NO_EXTRA_ERROR;
		int dataType = Constants.NO_EXTRA_ERROR;
		/*switch (choice) {
		case(0): 
			destination = Constants.DEST_BATTERYCONTROLLER;
			dataType = Constants.REQUEST_BATTERY_HEALTH;
			break;
		case(1): 
			destination = Constants.DEST_BATTERYCONTROLLER;
			dataType = Constants.REQUEST_BATTERY_LEVEL;
			break;
		case(2): 
			destination = Constants.DEST_BATTERYCONTROLLER;
			dataType = Constants.REQUEST_BATTERY_PLUGGED;
			break;
		case(3):  
			destination = Constants.DEST_BATTERYCONTROLLER;
			dataType = Constants.REQUEST_BATTERY_STATUS;
			break;
		case(4): 
			destination = Constants.DEST_BATTERYCONTROLLER;
			dataType = Constants.REQUEST_BATTERY_TEMPERATURE;
			break;
		case(5): 
			destination = Constants.DEST_CONNECTCONTROLLER;
			dataType = Constants.REQUEST_CONN_TYPE;
			break;
		case(6): 
			destination = Constants.DEST_CONNECTCONTROLLER;
			dataType = Constants.REQUEST_CONN_SUBTYPE;
			break;
		case(7): 
			destination = Constants.DEST_SCREENCONTROLLER;
			dataType = Constants.REQUEST_SCREEN_LOCK;
			break;
				}
*/
		
		//DESTINATION AND DATATYPE.
		
		dataType = this.ids.get(choice);
		Metric chosen = MetricsManager.metrics.get(dataType);
		CharSequence text = "";
		if (chosen!=null){
			text = this.ctx.getText(chosen.getStringID());
			destination = chosen.getControllerID();
		}
		
		//String text = (String) this.getItemsSequence()[choice];
		this.button.setText(text);
		this.data.setId(dataType);

		this.sendIntent(destination, dataType);
		
	}

	private void sendIntent(int destinationID, int dataType) {

		Intent startServiceIntent = new Intent(this.ctx, DataService.class);
		LogManager.print(Log.DEBUG, Constants.debugTag, "Activity: Requesting data.");
		
		this.data.setId(dataType);
        startServiceIntent.putExtra(Constants.EXTRAKEY_DATAREQUEST, Constants.BASICDATA_REQUEST);
        startServiceIntent.putExtra(Constants.EXTRAKEY_DESTINATION_ID, destinationID);
        startServiceIntent.putExtra(Constants.EXTRAKEY_DATATYPE, dataType);
        startServiceIntent.putExtra(Constants.EXTRAKEY_GRAPHNUMBER, graphNumber);

        
        
        ctx.startService(startServiceIntent);		
	}
	
	
	
	
}
