package com.moonapps.commandroid.networks;
/**
 * Copyright (C) 2012 Daniel Pavon Perez
 *  
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *  
 *      http://www.apache.org/licenses/LICENSE-2.0
 *  
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
import java.io.Serializable;

import android.os.Parcel;
import android.os.Parcelable;


public class WifiSupportUnit implements Serializable, Parcelable {

	private static final long serialVersionUID = -268815788001347533L;
	
	
	private boolean enabled;
	private boolean staticIPEnabled;

	private String name;
	private String ssid;
	private String bssid;
	
	private String ip;
	private String netmask;
	private String gateway;
	private String dns1;
	private String dns2;
	private int netSuccess;  //Success rate?
	private int preference;  //From 1 to 5.
	
	public WifiSupportUnit(){
		this.enabled=false;
		this.staticIPEnabled=false;
		this.name="";
		this.ssid="";
		this.bssid="";
		this.ip = "";
		this.netmask="";
		this.gateway="";
		this.dns1="";
		this.dns2="";
		this.preference=1;
		this.netSuccess=0;
	}
	
	
	public void setEnabled(boolean enabled) {
		this.enabled = enabled;
	}
	public void setName(String name) {
		this.name = name;
	}


	public String getName() {
		return name;
	}


	public void setBssid(String bssid) {
		this.bssid = bssid;
	}


	public String getBssid() {
		return bssid;
	}


	public void setSsid(String ssid) {
		this.ssid = ssid;
	}


	public String getSsid() {
		return ssid;
	}


	public boolean isEnabled() {
		return enabled;
	}
	public void setStaticIPEnabled(boolean staticIPEnabled) {
		this.staticIPEnabled = staticIPEnabled;
	}


	public boolean isStaticIPEnabled() {
		return staticIPEnabled;
	}


	public void setIp(String ip) {
		this.ip = ip;
	}
	public String getIp() {
		return ip;
	}
	public void setNetmask(String netmask) {
		this.netmask = netmask;
	}
	public String getNetmask() {
		return netmask;
	}
	public void setGateway(String gateway) {
		this.gateway = gateway;
	}
	public String getGateway() {
		return gateway;
	}
	public void setDns1(String dns1) {
		this.dns1 = dns1;
	}
	public String getDns1() {
		return dns1;
	}
	public void setDns2(String dns2) {
		this.dns2 = dns2;
	}
	public String getDns2() {
		return dns2;
	}
	public void setNetSuccess(int netSuccess) {
		this.netSuccess = netSuccess;
	}
	public int getNetSuccess() {
		return netSuccess;
	}
	public void setPreference(int preference) {
		this.preference = preference;
	}
	public int getPreference() {
		return preference;
	}
	
	
	
	//PARCELABLE METHODS
	

	@Override
	public int describeContents() {
		return 0;
	} 

	@Override 
	public void writeToParcel(Parcel out, int flags) {
		

        out.writeByte((byte)(this.isEnabled() ? 1 : 0));
        out.writeByte((byte)(this.isStaticIPEnabled() ? 1 : 0));

		out.writeString(this.getName());
        
		out.writeString(this.getSsid());
		out.writeString(this.getBssid());
        
		out.writeString(this.getIp());
        out.writeString(this.getNetmask());
		out.writeString(this.getGateway());
		out.writeString(this.getDns1());
		out.writeString(this.getDns2());
        out.writeInt(this.getNetSuccess());
        out.writeInt(this.getNetSuccess());

	}
	
	public static final Parcelable.Creator<WifiSupportUnit> CREATOR = new Parcelable.Creator<WifiSupportUnit>() {
		public WifiSupportUnit createFromParcel(Parcel in) {
			return new WifiSupportUnit(in);
		}

		public WifiSupportUnit[] newArray(int size) {
			return new WifiSupportUnit[size];
		}
	};
	
	private WifiSupportUnit(Parcel in) {
		
		this (
			in.readByte()== 1, 
			in.readByte()== 1, 
			
			in.readString(),
			
			in.readString(),
			in.readString(),
			
			in.readString(),
			in.readString(),
			in.readString(),
			in.readString(),
			in.readString(),
			in.readInt(), 
			in.readInt());
	}

	public WifiSupportUnit (boolean enabled, boolean staticIPEnabled, String name, String ssid, String bssid, 
			String ip, String netmask,	String gateway, String dns1, String dns2, 
			int netSuccess, int preference){
		this.enabled=enabled;
		this.staticIPEnabled=staticIPEnabled;
		this.name=name;
		this.ssid=ssid;
		this.bssid=bssid;
		this.ip=ip;
		this.netmask=netmask;
		this.gateway=gateway;
		this.dns1=dns1;
		this.dns2=dns2;
		this.netSuccess=netSuccess;
		this.preference=preference;
			
	}
	
	
}
