package com.moonapps.commandroid.profiles;
/**
 * Copyright (C) 2012 Daniel Pavon Perez
 *  
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *  
 *      http://www.apache.org/licenses/LICENSE-2.0
 *  
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
import java.io.Serializable;
import java.util.ArrayList; 
import java.util.Arrays;
import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.List;

import com.moonapps.commandroid.LogManager;
import com.moonapps.commandroid.agents.Constants;

import android.os.Parcel;
import android.os.Parcelable;
import android.util.Log;

public class Profile implements Serializable, Parcelable{
	
	
	private static final long serialVersionUID = -5561026392933889922L;
	
	private String name;
	private ArrayList<Trigger> triggers;
	private ArrayList<Action> actions;
	private boolean usingTimer;
	private boolean usingTriggers;
	private long milliDayStart;
	private long milliDayEnd;
	private boolean isActive;
	
	public Profile(){
		this.setName("");
		this.triggers = new ArrayList<Trigger> ();
		this.actions = new ArrayList<Action>();
		this.usingTimer=false;
		this.usingTriggers=false;
		this.milliDayEnd=0;
		this.milliDayStart=0;
		this.isActive=false;
	}
	
	public Profile (String name, List<Parcelable> trigger, List<Parcelable> action, 
			boolean hasTimer, boolean hasTriggers, long milliStart, 
			long milliEnd, boolean isActive){
		this.setName(name);
		this.triggers = new ArrayList<Trigger>();
		for (Parcelable e: trigger)
			this.triggers.add((Trigger) e);
		this.actions = new ArrayList<Action> ();
		for (Parcelable r: action)
			this.actions.add((Action) r);
		this.usingTimer=hasTimer;
		this.usingTriggers=hasTriggers;
		this.milliDayStart=milliStart;
		this.milliDayEnd=milliEnd;
		this.isActive=isActive;
		
	}
	
	
	@Override
	public int describeContents() {
		return 0;
	} 

	@Override 
	public void writeToParcel(Parcel out, int flags) {
		out.writeString(this.getName());
		Trigger[] trig = new Trigger[0];
		trig = this.triggers.toArray(trig);
        out.writeParcelableArray(trig, flags);
		Action[] acti =new Action[0];
		acti = this.actions.toArray(acti);
        out.writeParcelableArray(acti, flags);
        out.writeByte((byte)(this.usingTimer ? 1 : 0));
        out.writeByte((byte)(this.usingTriggers ? 1 : 0));  
        out.writeLong(this.milliDayStart);
        out.writeLong(this.milliDayEnd);
        out.writeByte((byte)(this.isActive ? 1 : 0));  

	}
	
	public static final Parcelable.Creator<Profile> CREATOR = new Parcelable.Creator<Profile>() {
		public Profile createFromParcel(Parcel in) {
			return new Profile(in);
		}

		public Profile[] newArray(int size) {
			return new Profile[size];
		}
	};

	private Profile(Parcel in) {
	
		this (in.readString(),Arrays.asList(in.readParcelableArray(Trigger.class.getClassLoader())), 
				Arrays.asList(in.readParcelableArray(Action.class.getClassLoader())), 
				in.readByte()== 1, 
				in.readByte()== 1, 
				in.readLong(), 
				in.readLong(),
				in.readByte()== 1);
	}

	public ArrayList<Trigger> getTriggers() {
		return triggers;
	}

	public void setTriggers(ArrayList<Trigger> triggers) {
		this.triggers = triggers;
	}

	public ArrayList<Action> getActions() {
		return actions;
	}

	public void setActions(ArrayList<Action> actions) {
		this.actions = actions;
	}

	public boolean isUsingTimer() {
		return usingTimer;
	}

	public void setUsingTimer(boolean usesTimer) {
		this.usingTimer = usesTimer;
	}

	public boolean isUsingTriggers() {
		return usingTriggers;
	}

	public void setUsingTriggers(boolean usesTriggers) {
		this.usingTriggers = usesTriggers;
	}

	public long getMilliDayStart() {
		return milliDayStart;
	}

	public void setMilliDayStart(long milliDayStart) {
		this.milliDayStart = milliDayStart;
	}

	public long getMilliDayEnd() {
		return milliDayEnd;
	}

	public void setMilliDayEnd(long milliDayEnd) {
		this.milliDayEnd = milliDayEnd;
	}

	public void setName(String name) {
		this.name = name;
	}

	public String getName() {
		return name;
	}

	
	@Override
	public boolean equals(Object obj) {
	      
		 if(this == obj) return true;

	      if( !(obj instanceof Profile) ) return false;

	       Profile p = (Profile)obj;

	       boolean equals = (this.getMilliDayEnd() == p.getMilliDayEnd());
	       equals = equals && (this.getMilliDayStart() == p.getMilliDayStart());
	       equals = equals && (this.isUsingTimer() == p.isUsingTimer());
	       equals = equals && (this.isUsingTriggers() == p.isUsingTriggers());
	       equals = equals && (this.getName().equals(p.getName()));
	       equals = equals && (this.getActions().equals(p.getActions()));
	       equals = equals && (this.getTriggers().equals(p.getTriggers()));
	       equals = equals && (this.isActive() == p.isActive());
	       return equals;
	   }

	public boolean isTimeTriggerable() {
		
		if (this.isUsingTimer()==false)
			return true;
		
		Date now = new Date (((long) System.currentTimeMillis())+80000L);

		GregorianCalendar cal = (GregorianCalendar) GregorianCalendar.getInstance();
    	cal.setTime(now);
    	cal.set(Calendar.MINUTE, 0);
    	cal.set(Calendar.HOUR_OF_DAY, 0);
    	
    	Date beginning = new Date(cal.getTimeInMillis()+this.getMilliDayStart());
    	Date end = new Date(cal.getTimeInMillis()+this.getMilliDayEnd());

    	LogManager.print(Log.VERBOSE, Constants.debugTag, "Profile:"+this.getName()+
    			" Begin:"+beginning.toLocaleString()+
    			" Now:"+now.toLocaleString()+" End:"+end.toLocaleString());
		if (now.after(beginning) && now.before(end))
			return true;
		
		return false;
	}

	public void setActive(boolean isActive) {
		this.isActive = isActive;
	}

	public boolean isActive() {
		return isActive;
	}

	
}
