package com.moonapps.commandroid.profiles;
/**
 * Copyright (C) 2012 Daniel Pavon Perez
 *  
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *  
 *      http://www.apache.org/licenses/LICENSE-2.0
 *  
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;

import android.app.AlarmManager;
import android.app.PendingIntent;
import android.content.Context;
import android.content.Intent;
import android.os.Bundle;

import com.moonapps.commandroid.DataService;
import com.moonapps.commandroid.agents.Constants;


public class ProfileAlarmManager {

		
    public static void createAlarm (Context ctx, Profile profile,long time, boolean isStart){
    	
		Intent intent = new Intent(ctx,
                DataService.class);

		int reqCode = (int)time;
		
		Bundle bundle = new Bundle();
		bundle.putParcelable("profile", profile);
		bundle.putInt("reqCode", reqCode);
		bundle.putBoolean("isStart", isStart);

		intent.putExtra(Constants.BUNDLEKEY, bundle);
		intent.putExtra(Constants.ALARM, true);
		
		PendingIntent pintent = PendingIntent.getService(
				ctx, reqCode, 
                intent, 0);

		AlarmManager alarm = 
			(AlarmManager) ctx.getSystemService(Context.ALARM_SERVICE);

		alarm.set(AlarmManager.RTC_WAKEUP, 
				time, pintent);
    }
    
    public static void cancelAlarm (Context ctx, Profile profile, long time, boolean isStart){
    	
		int reqCode = (int)time;

    	
		Intent intent = new Intent(ctx, DataService.class);
		
		Bundle bundle = new Bundle();
		bundle.putParcelable("profile", profile);
		bundle.putInt("reqCode", reqCode);
		bundle.putBoolean("isStart", isStart);

		intent.putExtra(Constants.BUNDLEKEY, bundle);
		intent.putExtra(Constants.ALARM, true);

		PendingIntent pintent = PendingIntent.getService(
				ctx, reqCode, 
                intent, 0);

		AlarmManager alarm = 
			(AlarmManager) ctx.getSystemService(Context.ALARM_SERVICE);

		alarm.cancel(pintent);
    }
	
	
	
    public static Date getNextTime(long time){
    	Date result = null;
    	
    	GregorianCalendar cal = (GregorianCalendar) GregorianCalendar.getInstance();
    	
    	Date now = new Date(System.currentTimeMillis());
    	cal.setTime(now);
    	
    	cal.set(Calendar.MINUTE, 0);
    	cal.set(Calendar.HOUR_OF_DAY, 0);
    	Date expected = new Date(cal.getTimeInMillis()+time);
    	
    	if (now.before(expected))
    		result = expected;
    	else{
    		cal.setTime(expected);
    		cal.add(Calendar.DAY_OF_YEAR, 1);
    		result = cal.getTime();
    	} 
    		
    	return result;
    }
	
	
}
